/*
 * Created on Nov 22, 2004
 */
package interview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.JTextComponent;

import com.sun.media.jai.widget.DisplayJAI;

import ui.StackLayout;
import ch.randelshofer.quaqua.QuaquaManager;

/**
 * @author ronyeh Apparently WAV files have a 44 byte header....
 */
public class Annotater {

	private class OpenAction extends AbstractAction {

		public OpenAction() {
			super("Open");
			putValue(SHORT_DESCRIPTION, "Open Audio File");
			putValue(MNEMONIC_KEY, new Integer(KeyEvent.VK_O));
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			int returnVal = fileChooser.showOpenDialog(mainFrame);
			if (returnVal == JFileChooser.APPROVE_OPTION) { // file loaded
				File file = fileChooser.getSelectedFile();
				fileLoaded.setText(file.getName());
				loadSound(file);
				loadOrCreateAnnotationFile();
				enableWidgets_SoundFileLoaded();
			}
		}
	}

	private static final String ADD_STR = "Add Note";

	private static final String ADD_STR_M5 = "Add Note (-5s)";

	private static final String BACK_STR = "< Back 5s";

	private static final Calendar CALENDAR = new GregorianCalendar();

	private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(
			"MMMM dd, yyyy");

	private static final String FILE_STR = "Current Sound File";

	private static final String FWD_STR = "Forward 5s >";

	private static final String NOFILE_STR = "None Open";

	private static final String NXT_STR = "Next >>";

	private static final String OPEN_STR = "Open ";

	private static final String PAUSE_AND_ADD_STR = "Pause and Add Note";

	private static final String PAUSE_STR = "Pause ||";

	private static final int PAUSED = 0;

	private static final String PLAY_STR = "Play |>";

	private static final int PLAYING = 1;

	private static final String PREV_STR = "<< Previous";

	private static final String SAVE_STR = "Save";

	private static final boolean SHOW_COMPONENT_BOUNDARIES = false;

	private static final String START_STR = "|<< Start";

	private static final String STOP_STR = "Stop [ ]";

	private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat(
			"hh:mm:ss a");

	private static final String TITLE_STR = "Parrot Interview Annotater";

	private static final Toolkit TOOLKIT = Toolkit.getDefaultToolkit();

	// entry
	public static void main(String[] args) {

		// check java properties
		System.out.println(System.getProperty("java.library.path"));

		try {
			UIManager.setLookAndFeel(QuaquaManager.getLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Quaqua didn't load...");
		}
		Annotater annotater = new Annotater();
	}

	private JButton addNote = null;

	private StringBuffer annotationBuffer = new StringBuffer();

	private Cursor audioCursor = null;

	/**
	 * Progress Meter displaying where we are in the song... Can be controlled.
	 */
	private JSlider audioSlider = null;

	// whenever the slider is touched, flag this, so that we can start
	// playing the audio at the correct moment...
	private boolean audioSliderChanged = false;

	private JButton back5 = new JButton(BACK_STR);

	private JPanel buttonPanel = new JPanel();

	private int consecutiveEnterCount = 0;

	private JPanel controlPanel = new JPanel();

	/**
	 * Used to temporarily store the most currently applicable date.
	 */
	private Date currentDate = CALENDAR.getTime();

	private JTextField date = null;

	private JFileChooser fileChooser = new JFileChooser();

	private JTextField fileLoaded = new JTextField(NOFILE_STR);

	private JLabel fileLoadedLabel = new JLabel(FILE_STR);

	private NumberFormat format = NumberFormat.getInstance();

	private JButton fwd5 = new JButton(FWD_STR);

	private Cursor helpCursor = null;

	private JFrame mainFrame = new JFrame(TITLE_STR);

	private JButton next = new JButton(NXT_STR);

	private JTextArea notesArea = null;

	private JPanel notesPanel = new JPanel();

	private JScrollPane notesScroll = new JScrollPane(getNotesArea());

	private JButton open = null;

	private Action openAction = null;

	private JButton pauseAndAddNote = null;

	/**
	 * Play/Pause Button
	 */
	private JButton play_pause = null;

	private int play_pause_toggle = PAUSED;

	private JPanel playerPanel = new JPanel();

	private JPopupMenu popup = null;

	private JTextField rate = new JTextField("1.0", 4);

	// how fast to play the audio? 0 -> .66 speed, 34 -> 1X speed, 67 -> 1.33x
	private JSlider rateSlider = new JSlider(JSlider.HORIZONTAL, 0, 134, 34);

	private JButton save = null;

	private Action selectLine = null;

	private SoundControl soundControl = null;

	private JButton start_prev = new JButton(START_STR);

	private JButton stop = null;

	private boolean temporaryStop = false;

	private JTextField time = null;

	private JPanel timePanel = null;

	private JButton addNoteM5;

	private JTextField gain = null;

	private JSlider gainSlider = null;

	public Annotater() {
		fileChooser.setDialogTitle("Open Sound File");
		fileChooser.setCurrentDirectory(new File("."));
		setupGUI();
		disableWidgets_SoundFileNotLoaded();

		selectLine = getAction(DefaultEditorKit.selectLineAction);

		{ // cursor 1
			Image image = TOOLKIT.getImage("cursors/audioCursor-7-6.gif");
			audioCursor = TOOLKIT.createCustomCursor(image, new Point(7, 6),
					"img");
		}

		{ // cursor 2
			Image image = TOOLKIT.getImage("cursors/helpCursor-6-6.gif");
			helpCursor = TOOLKIT.createCustomCursor(image, new Point(6, 6),
					"img");
		}
	}

	private void addNoteAction() {
		int cursorPosition = notesArea.getCaretPosition();
		String timeStamp = getCurrentTimeStamp();
		notesArea.insert(timeStamp, cursorPosition);
		notesArea.requestFocus();
	}

	/**
	 * Gray out some widgets when no sound file is loaded.
	 */
	public void disableWidgets_SoundFileNotLoaded() {
		back5.setEnabled(false);
		fwd5.setEnabled(false);
		next.setEnabled(false);
		play_pause.setEnabled(false);
		stop.setEnabled(false);
		start_prev.setEnabled(false);
		audioSlider.setEnabled(false);
		addNote.setEnabled(false);
		save.setEnabled(false);
		pauseAndAddNote.setEnabled(false);
		addNoteM5.setEnabled(false);
	}

	/**
	 *  
	 */
	public void enableWidgets_SoundFileLoaded() {
		back5.setEnabled(true);
		fwd5.setEnabled(true);
		next.setEnabled(true);
		play_pause.setEnabled(true);
		stop.setEnabled(true);
		start_prev.setEnabled(true);
		audioSlider.setEnabled(true);
		addNote.setEnabled(true);
		save.setEnabled(true);
		pauseAndAddNote.setEnabled(true);
		addNoteM5.setEnabled(true);
	}

	/**
	 * @param selectLineAction
	 * @return
	 */
	private Action getAction(String name) {
		Action action = null;
		Action[] actions = getNotesArea().getActions();
		for (int i = 0; i < actions.length; i++) {
			if (name.equals(actions[i].getValue(Action.NAME).toString())) {
				action = actions[i];
				break;
			}
		}
		return action;
	}

	/**
	 * @return
	 */
	private JButton getAddNote() {
		if (addNote == null) {
			addNote = new JButton(ADD_STR);
			addNote.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					addNoteAction();
				}
			});
		}
		return addNote;
	}

	private JButton getAddNoteMinus5() {
		if (addNoteM5 == null) {
			addNoteM5 = new JButton(ADD_STR_M5);
			addNoteM5.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					addNoteM5Action();
				}
			});
		}
		return addNoteM5;
	}

	/**
	 *  
	 */
	protected void addNoteM5Action() {
		int cursorPosition = notesArea.getCaretPosition();
		String timeStamp = getCurrentTimeStamp(-5);
		notesArea.insert(timeStamp, cursorPosition);
		notesArea.requestFocus();
	}

	/**
	 * @param offset
	 * @return
	 */
	private String getCurrentTimeStamp(int offset) {
		int numRelativeSeconds = audioSlider.getValue() + offset;
		long base = soundControl.getBaseDateInMilliseconds();
		long totalMilliseconds = base + numRelativeSeconds * 1000;
		currentDate = new Date(totalMilliseconds);
		return "[" + currentDate.toString() + "] ::" + numRelativeSeconds
				+ ":: ";
	}

	/**
	 * audio slider (where you are in the sound file)
	 * 
	 * @return
	 */
	private JSlider getAudioSlider() {
		if (audioSlider == null) {
			audioSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 0);
			audioSlider.addMouseMotionListener(new MouseMotionListener() {

				public void mouseDragged(MouseEvent e) {
					updateTimeDisplay();
				}

				public void mouseMoved(MouseEvent e) {

				}

			});
			audioSlider.addMouseListener(new MouseListener() {

				public void mouseClicked(MouseEvent e) {
				}

				public void mouseEntered(MouseEvent e) {
				}

				public void mouseExited(MouseEvent e) {
				}

				public void mousePressed(MouseEvent e) {
					if (play_pause_toggle == PLAYING) {
						pauseButtonAction();
						temporaryStop = true;
					}
					audioSliderChanged = true;
				}

				public void mouseReleased(MouseEvent e) {
					if (temporaryStop) {
						temporaryStop = false;
						playButtonAction();
					}
				}

			});
		}
		return audioSlider;
	}

	/**
	 * @return A time stamp that's 5 seconds before when you ask for it... to
	 *         compensate for human lag and typing time... :o) Empirically
	 *         determined, of course.
	 */
	private String getCurrentAbbreviatedTimeStamp() {
		return "::" + Math.max(0, audioSlider.getValue() - 5) + ":: ";
	}

	/**
	 * @return the most current real date/time (as we are playing the sound
	 *         file)
	 */
	private String getCurrentTimeStamp() {
		return getCurrentTimeStamp(0);
	}

	public JTextField getDate() {
		if (date == null) {
			date = new JTextField(DATE_FORMAT.format(currentDate));
			date.setEditable(false);
		}
		return date;
	}

	/**
	 * @return The TextArea to write annotations in.
	 */
	private JTextArea getNotesArea() {
		if (notesArea == null) {
			notesArea = new JTextArea(25, 60);
			notesArea.setWrapStyleWord(true);
			notesArea.setLineWrap(true);
			notesArea.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createMatteBorder(0 /* top */,
							15 /* left */, 0/* bottom */, 0/* right */,
							new Color(220, 230, 230)), BorderFactory
							.createEmptyBorder(15, 5, 15, 20)));

			notesArea.addKeyListener(new KeyListener() {

				public void keyPressed(KeyEvent e) {
					int keyCode = e.getKeyCode();

					if (keyCode == KeyEvent.VK_ENTER) {
						consecutiveEnterCount++;
					} else if (keyCode == KeyEvent.VK_SHIFT) {
						// do nothing
					} 
					else if (keyCode == KeyEvent.VK_ESCAPE) {
						// reset the counter!
						consecutiveEnterCount = 0;
					}
					else { // some random character
						if (consecutiveEnterCount >= 2) {
							// insert the current abbreviated time stamp
							int cursorPosition = notesArea.getCaretPosition();
							String timeStamp = getCurrentAbbreviatedTimeStamp();
							notesArea.insert(timeStamp, cursorPosition);
							notesArea.requestFocus();
						}
						consecutiveEnterCount = 0; // reset it
					}
					
					if (consecutiveEnterCount >= 2) {
						notesArea.setCaretColor(Color.ORANGE);
					}
					else {
						notesArea.setCaretColor(null); // restore the color
					}
				}

				public void keyReleased(KeyEvent e) {
				}

				public void keyTyped(KeyEvent e) {
				}
			});

			notesArea.addMouseMotionListener(new MouseMotionListener() {

				public void mouseDragged(MouseEvent e) {
					if (e.getX() < 13) {
						notesArea.setCursor(audioCursor);
					} else {
						notesArea.setCursor(Cursor
								.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				}

				public void mouseMoved(MouseEvent e) {
					if (e.getX() < 13) {
						notesArea.setCursor(audioCursor);
					} else {
						notesArea.setCursor(Cursor
								.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				}

			});
			notesArea.addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent e) {

				}

				public void mouseEntered(MouseEvent e) {

				}

				public void mouseExited(MouseEvent e) {

				}

				public void mousePressed(MouseEvent e) {
				}

				public void mouseReleased(MouseEvent e) {
					if (e.getX() < 13) {
						selectLine.actionPerformed(null);

						// parse this line and jump to the correct time
						jumpToTimeInString(notesArea.getSelectedText());
					}
				}

			});
		}
		return notesArea;
	}

	public Action getOpenAction() {
		if (openAction == null) {
			openAction = new OpenAction();
		}
		return openAction;
	}

	/**
	 * @return
	 */
	private JButton getOpenButton() {
		if (open == null) {
			open = new JButton(getOpenAction());
		}
		return open;
	}

	/**
	 * @return
	 */
	private JButton getPauseAndAddNote() {
		if (pauseAndAddNote == null) {
			pauseAndAddNote = new JButton(PAUSE_AND_ADD_STR);
			pauseAndAddNote.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					pauseButtonAction();
					addNoteAction();
				}

			});
		}
		return pauseAndAddNote;
	}

	/**
	 * @return
	 */
	private JButton getPlayPause() {
		if (play_pause == null) {
			play_pause = new JButton(PLAY_STR);
			play_pause.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (play_pause_toggle == PAUSED) { // currently paused,
						// so...
						playButtonAction();
					} else { // stop the sound temporarily
						pauseButtonAction();
					}
				}
			});
		}
		return play_pause;
	}

	/**
	 * @return
	 */
	private JPopupMenu getPopup() {
		if (popup == null) {
			popup = new JPopupMenu();
			JMenuItem jumpTo = new JMenuItem("Synchronize Audio to this line");
			popup.add(jumpTo);
			jumpTo.addActionListener(new ActionListener() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
				 */
				public void actionPerformed(ActionEvent e) {
					System.out.println("Synchronizing Audio");

				}
			});
		}
		return popup;
	}

	/**
	 * @return the save button
	 */
	private JButton getSave() {
		if (save == null) {
			save = new JButton(SAVE_STR);
			save.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					// get text from the notes area
					String text = notesArea.getText();

					// copy the old file to *.bak
					String annotationFilePath = soundControl.getFilePath()
							+ ".txt";
					File annotationFile = new File(annotationFilePath);
					if (annotationFile.exists()) { // CREATE NEW ANNOTATION FILE
						File bak = new File(annotationFilePath + ".bak");
						if (bak.exists()) {
							bak.delete();
						}
						annotationFile.renameTo(bak);
					} else {
						// nothing
					}

					try {
						// create the new file and dump in the text...
						annotationFile.createNewFile();
						FileOutputStream fos = new FileOutputStream(
								annotationFile);
						BufferedOutputStream bos = new BufferedOutputStream(fos);
						PrintWriter pw = new PrintWriter(bos);
						pw.print(text);
						pw.flush();
						pw.close();
						fos.flush();
						fos.close();
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
			});
		}
		return save;
	}

	/**
	 * @return
	 */
	private JButton getStop() {
		if (stop == null) {
			stop = new JButton(STOP_STR);
			stop.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					stopButtonAction();
				}
			});
		}
		return stop;
	}

	/**
	 * @return the time display widget
	 */
	public JTextField getTime() {
		if (time == null) {
			time = new JTextField(TIME_FORMAT.format(currentDate));
			time.setEditable(false);
		}
		return time;
	}

	public JPanel getTimePanel() {
		if (timePanel == null) {
			timePanel = new JPanel();
			timePanel.setLayout(new StackLayout(StackLayout.VERTICAL));

			JPanel p1 = new JPanel();
			p1.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			p1.add(getDate(), "Wide");
			if (SHOW_COMPONENT_BOUNDARIES) {
				p1.setBorder(BorderFactory.createLineBorder(Color.RED));
			}

			JPanel p2 = new JPanel();
			p2.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			p2.add(getTime(), "Wide");

			timePanel.add(p1, "Wide");
			timePanel.add(p2, "Wide");
		}
		return timePanel;
	}

	/**
	 * @param selectedText
	 */
	protected void jumpToTimeInString(String selectedText) {
		System.out.println("Jumping to " + selectedText);

		Pattern p = Pattern.compile("::(\\d+)::");
		Matcher m = p.matcher(selectedText);
		boolean b = m.find();
		if (b) {
			String jumpTo = m.group(1);

			if (play_pause_toggle == PLAYING) {
				stopButtonAction();
			}
			moveAudioSliderTo(Integer.parseInt(jumpTo));
			playButtonAction();
		}
	}

	/**
	 *  
	 */
	private void loadOrCreateAnnotationFile() {
		// load or create associated annotation file
		String annotationFilePath = soundControl.getFilePath() + ".txt";
		File annotationFile = new File(annotationFilePath);
		if (!annotationFile.exists()) { // CREATE NEW ANNOTATION FILE
			System.out.println("New File!");
			try {
				annotationFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
			annotationBuffer = new StringBuffer();
		} else { // LOAD OLD ANNOTATION FILE
			System.out.println("Loading Old File!");
			// load in the text...
			annotationBuffer = new StringBuffer();
			BufferedReader input = null;
			try {
				input = new BufferedReader(new FileReader(annotationFile));
				String line = null;
				while ((line = input.readLine()) != null) {
					annotationBuffer.append(line);
					annotationBuffer.append(System
							.getProperty("line.separator"));
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		// load up the text into the notes area
		notesArea.setText(annotationBuffer.toString());
	}

	/**
	 * Opens a file for listening...
	 */
	private void loadSound(File file) {
		soundControl = new SoundControl(file, this);
		System.out.println("Number Bytes in Stream... "
				+ soundControl.getTotalBytesInStream());

		// update slider....
		audioSlider.setMaximum(soundControl.getLengthInSeconds());
	}

	private void pauseButtonAction() {
		System.out.println("pauseButtonAction()");

		play_pause_toggle = PAUSED;
		soundControl.setPaused(true);
		play_pause.setText(PLAY_STR);
	}

	private void playButtonAction() {
		System.out.println("playButtonAction()");

		play_pause_toggle = PLAYING;
		play_pause.setText(PAUSE_STR);

		if (audioSliderChanged) {
			audioSliderChanged = false; // clean the flag
			soundControl.playFrom(audioSlider.getValue());
			System.out.println("Playing From " + audioSlider.getValue());
		} else {
			soundControl.play();
		}
	}

	/**
	 *  
	 */
	public void resetAudioSlider() {
		audioSlider.setValue(0);
	}

	/**
	 * @param pct
	 *            Used by the SoundControl thread to update the audio slider...
	 */
	public void setProgress(int numSeconds) {
		audioSlider.setValue(numSeconds);
		updateTimeDisplay();
	}

	/**
	 * For "manual" changes in the code.
	 * 
	 * @param numSeconds
	 */
	public void moveAudioSliderTo(int numSeconds) {
		audioSliderChanged = true;
		audioSlider.setValue(numSeconds);
		updateTimeDisplay();
	}

	/**
	 *  
	 */
	private void setupGUI() {
		// mainFrame
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.setSize(1100, 800);

		JPanel top = new JPanel();
		JPanel bottom = new JPanel();
		top.setLayout(new BorderLayout());
		bottom.setLayout(new BorderLayout());

		top.add(controlPanel, BorderLayout.NORTH);
		top.add(playerPanel, BorderLayout.SOUTH);
		bottom.add(notesPanel, BorderLayout.CENTER);
		bottom.add(buttonPanel, BorderLayout.SOUTH);

		// rate slider
		{
			rate.setEditable(false);
			format.setMaximumFractionDigits(2);
			rateSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					int value = rateSlider.getValue();
					float rateVal = ((float) (value - 34)) / 100f + 1.0f;
					rate.setText(format.format(rateVal));
					if (soundControl != null) {
						soundControl.setRate(rateVal);
					}
				}
			});
		}

		// playerPanel
		playerPanel.setLayout(new BorderLayout());
		playerPanel.add(getAudioSlider(), BorderLayout.CENTER);

		// controlpanel
		{ // set up buttons

			controlPanel.add(start_prev);
			controlPanel.add(back5);
			controlPanel.add(getPlayPause());
			controlPanel.add(getStop());
			controlPanel.add(getOpenButton());
			controlPanel.add(fwd5);
			controlPanel.add(next);

			JPanel p = new JPanel();
			p.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			p.add(new JLabel("Speed"), "Left");
			p.add(rateSlider, "LeftWide");
			p.add(rate, "LeftWidth=40");
			p.setBorder(BorderFactory.createEtchedBorder());
			controlPanel.add(p);
		}

		// buttonPanel
		{
			buttonPanel.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			buttonPanel.add(getAddNote(), "Left");
			buttonPanel.add(getPauseAndAddNote(), "Left");
			buttonPanel.add(getAddNoteMinus5(), "Left");
			buttonPanel.add(getSave(), "Left");
			fileLoaded.setEditable(false);
			fileLoaded.setColumns(32);

			JPanel temp = new JPanel();
			temp.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			temp.add(fileLoadedLabel);
			temp.add(fileLoaded, "Wide");

			buttonPanel.add(temp, "Width=300Right");
		}

		// notesPanel
		notesPanel.setLayout(new BorderLayout());
		notesPanel.add(notesScroll, BorderLayout.CENTER);

		JPanel leftPane = new JPanel();
		{ // left pane setup
			leftPane.setLayout(new BorderLayout());
			leftPane.add(top, BorderLayout.NORTH);
			leftPane.add(bottom, BorderLayout.CENTER);
		}

		JPanel rightPane = new JPanel();
		{ // right pane setup
			rightPane.setLayout(new StackLayout(StackLayout.VERTICAL));
			rightPane.add(getTimePanel(), "TopWide");

			JPanel p = new JPanel();
			p.setLayout(new StackLayout(StackLayout.HORIZONTAL));
			p.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createEtchedBorder(), "Gain"));
			p.add(getGainSlider(), "WideTall");
			p.add(getGain(), "Width=40");
			rightPane.add(p, "TopWide");

			DisplayJAI djLogo = new DisplayJAI();
			PlanarImage img = JAI.create("fileload", "images/PARROT.gif");
			djLogo.set(img);
			rightPane.add(djLogo, "Width=100 Height=100 Bottom");

			if (SHOW_COMPONENT_BOUNDARIES) {
				rightPane.setBorder(BorderFactory.createLineBorder(Color.RED));
			}
		}

		// add everything to the main frame
		Container content = mainFrame.getContentPane();
		content.setLayout(new StackLayout(StackLayout.HORIZONTAL));
		content.add(leftPane, "LeftWideTall");
		content.add(rightPane, "TallTopWidth=150");
		mainFrame.setVisible(true);
	}

	private JTextComponent getGain() {
		if (gain == null) {
			gain = new JTextField("0");
			gain.setColumns(4);
			gain.setEditable(false);
		}
		return gain;
	}

	/**
	 * @return
	 */
	private JSlider getGainSlider() {
		if (gainSlider == null) {
			gainSlider = new JSlider(JSlider.HORIZONTAL, -5, 10, 0);
			gainSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					System.out.println(gainSlider.getValue());
					int value = gainSlider.getValue();
					gain.setText(Integer.toString(value));
					if (soundControl != null) {
						soundControl.setGain((float) value);
					}
				}
			});
		}
		return gainSlider;
	}

	/**
	 *  
	 */
	public void stopButtonAction() {
		System.out.println("stopButtonAction()");
		play_pause.setText(PLAY_STR);
		soundControl.stop();
		play_pause_toggle = PAUSED;
		resetAudioSlider();
		updateTimeDisplay();
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Reflect the current audio bar in the time display...
	 */
	private void updateTimeDisplay() {
		getCurrentTimeStamp();
		getDate().setText(DATE_FORMAT.format(currentDate));
		getTime().setText(TIME_FORMAT.format(currentDate));
	}
}