/*
 * Created on Nov 21, 2004
 */
package interview;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * @author ronyeh
 *  
 */
public class SoundControl {

	private static int EXTERNAL_BUFFER_SIZE = 16000; // what units?

	private AudioFormat audioFormat = null;

	private AudioInputStream audioInputStream = null;

	private GregorianCalendar baseCalendar = null;

	private Date baseDate = null;

	/**
	 * Each sound file has a base time (create date/time) that reflects when the
	 * file was created. For example, if I recorded something on my iPod at Dec
	 * 25, 2004, 7:15:03 am, then the base time would reflect that, except that
	 * it'd be expressed in seconds.
	 */
	private int baseTimeInSeconds = 0;

	private String fileName;

	private String filePath;

	private long frameLength;

	private float frameRate;

	private int frameSize;

	private float lastRateVal = 1.0f;

	private int lengthInSeconds = 0;

	private SourceDataLine line = null;

	private Annotater parent = null;

	private boolean paused = true;

	private int playingFrom = 0;

	private FloatControl rateControl = null;

	private boolean reset;

	private float sampleRate;

	private File soundFile = null;

	private long totalBytesInStream;

	private int totalBytesRead;

	private FloatControl gainControl;

	private float lastGainVal = 0f;
	

	/**
	 * Does not work with MP3s yet...
	 * 
	 * @param soundFile
	 */
	public SoundControl(File soundFile, Annotater parent) {
		this.parent = parent;
		this.soundFile = soundFile;

		// calculate base time and date
		// fileName = 20041117 174919.wav
		filePath = soundFile.getAbsolutePath();
		fileName = soundFile.getName();

		try { // standard iPod recorder format
			baseTimeInSeconds = 0;

			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HHmmss");
			sdf.setCalendar(Calendar.getInstance());
			baseDate = sdf.parse(fileName.substring(0, fileName.indexOf(".")));

			System.out.println(baseDate.toString());
			baseCalendar = new GregorianCalendar();
			baseCalendar.setTime(baseDate);
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (ParseException e) {
			e.printStackTrace();
		}

		initialize();
	}

	public int getBaseDate() {
		return baseCalendar.get(Calendar.DAY_OF_MONTH);
	}

	public long getBaseDateInMilliseconds() {
		return baseDate.getTime();
	}

	public int getBaseHour() {
		return baseCalendar.get(Calendar.HOUR_OF_DAY);
	}

	public int getBaseMinute() {
		return baseCalendar.get(Calendar.MINUTE);
	}

	public int getBaseMonth() {
		return baseCalendar.get(Calendar.MONTH);
	}

	public int getBaseSecond() {
		return baseCalendar.get(Calendar.SECOND);
	}

	public int getBaseYear() {
		return baseCalendar.get(Calendar.YEAR);
	}

	/**
	 * @return int -- where we are in the audio file (in seconds)
	 */
	public int getCurrentPlayingLocationInSeconds() {
		// num Frames that we have read...
		float numFrames = (float) totalBytesRead / (float) frameSize;
		// num Seconds that we have passed
		float numSecs = numFrames / frameRate;
		// from the starting point
		return playingFrom + (int) numSecs;
	}

	/**
	 * @return Returns the filePath.
	 */
	public String getFilePath() {
		return filePath;
	}

	/**
	 * @return
	 */
	public int getLengthInSeconds() {
		return lengthInSeconds;
	}

	/**
	 * @return Returns the totalBytesInStream.
	 */
	public long getTotalBytesInStream() {
		return totalBytesInStream;
	}

	/**
	 * @return Returns the totalBytesRead.
	 */
	public int getTotalBytesRead() {
		return totalBytesRead;
	}

	/**
	 * Initialize stuff... :)
	 */
	private void initialize() {
		playingFrom = 0;

		try {
			audioInputStream = AudioSystem.getAudioInputStream(soundFile);
		} catch (UnsupportedAudioFileException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		audioFormat = audioInputStream.getFormat();
		DataLine.Info info = new DataLine.Info(SourceDataLine.class,
				audioFormat);

		try {
			line = (SourceDataLine) AudioSystem.getLine(info);
			line.open(audioFormat);
		} catch (LineUnavailableException e1) {
			e1.printStackTrace();
		}

		rateControl = (FloatControl) line
				.getControl(FloatControl.Type.SAMPLE_RATE);
		gainControl = (FloatControl) line.getControl(FloatControl.Type.MASTER_GAIN);
		
		frameSize = audioFormat.getFrameSize();
		frameRate = audioFormat.getFrameRate();
		sampleRate = audioFormat.getSampleRate();
		frameLength = audioInputStream.getFrameLength();

		totalBytesRead = 0;

		System.out.println("Frame Size:" + frameSize);
		System.out.println("Frame Rate:" + frameRate);
		System.out.println("Sample Rate: " + sampleRate);
		System.out.println("Frame Length: " + frameLength);

		totalBytesInStream = frameLength * frameSize;

		System.out.println("Total Bytes: " + totalBytesInStream);
		EXTERNAL_BUFFER_SIZE = (int) (2 * sampleRate);

		lengthInSeconds = (int) Math.ceil(frameLength / frameRate);

	}

	/**
	 *  
	 */
	public synchronized void initStopFlags() {
		reset = false;
		paused = false;
	}

	public synchronized boolean isPaused() {
		return paused;
	}

	/**
	 * @return
	 */
	private synchronized boolean isReset() {
		return reset;
	}

	public void play() {
		runPlayThread();
	}

	/**
	 * @param value
	 */
	public void playFrom(int numSeconds) {
		initialize();
		playingFrom = numSeconds;
		parent.setProgress(numSeconds);
		try {
			// skip numSeconds
			audioInputStream
					.skip((long) (numSeconds * frameRate * frameSize /* totalBytesToSkip */)); // num
			// bytes
		} catch (IOException e) {
			e.printStackTrace();
		}
		runPlayThread();
	}

	public void runPlayThread() {
		// reinit the state...
		initStopFlags();

		line.start();

		// NOTE: setting PAN can move between left and right ears
		// setting rate controls the speed
		rateControl.setValue((float) (sampleRate * lastRateVal));
		gainControl.setValue(lastGainVal); 
		
		
		// the player thread
		Runnable r = new Runnable() {
			public void run() {
				int nBytesRead = 0;
				byte[] abData = new byte[EXTERNAL_BUFFER_SIZE];

				while (nBytesRead != -1) {

					// did we stop fully?
					if (isReset()) {
						line.stop();
						line.flush();
						line.close();
						System.out.println("Stopped...");
						initialize();
						return;
					}

					// check the paused flag
					if (isPaused()) {
						line.stop();
						try {
							Thread.sleep(150);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						continue;
					}

					try {
						nBytesRead = audioInputStream.read(abData, 0,
								abData.length);
						if (nBytesRead >= 0) {
							totalBytesRead += nBytesRead;

							// update the GUI...
							parent
									.setProgress(getCurrentPlayingLocationInSeconds());

							int nBytesWritten = line.write(abData, 0,
									nBytesRead);
							try {
								// sleep a bit
								Thread.sleep(80);
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}
				line.drain();
				line.close();
				// press the stop button
				parent.stopButtonAction();
				initialize();
			}

		};

		Thread t = new Thread(r);
		t.start();

	}

	// pauses the audio playback
	public synchronized void setPaused(boolean stop) {
		paused = stop;
	}

	/**
	 * @param rateVal
	 */
	public void setRate(float rateVal) {
		lastRateVal = rateVal;
		rateControl.setValue((float) (sampleRate * rateVal));
	}

	// stop and move to the beginning of the recording...
	public synchronized void stop() {
		paused = true;
		reset = true;
	}

	/**
	 * @param gainVal
	 */
	public void setGain(float gainVal) {
		lastGainVal = gainVal;
		gainControl.setValue((float) gainVal);
	}
}